@@ -119,6 +119,36 @@ static int __create_ssl(SSL_CTX *ssl_ctx, struct CommConnEntry *entry)
 	return -1;
 }
 
+#define SSL_WRITE_BUFSIZE	8192
+
+static int __ssl_writev(SSL *ssl, const struct iovec vectors[], int cnt)
+{
+	char buf[SSL_WRITE_BUFSIZE];
+	size_t nleft = SSL_WRITE_BUFSIZE;
+	char *p = buf;
+	size_t n;
+	int i;
+
+	if (vectors[0].iov_len >= SSL_WRITE_BUFSIZE || cnt == 1)
+		return SSL_write(ssl, vectors[0].iov_base, vectors[0].iov_len);
+
+	for (i = 0; i < cnt; i++)
+	{
+		if (vectors[i].iov_len <= nleft)
+			n = vectors[i].iov_len;
+		else
+			n = nleft;
+
+		memcpy(p, vectors[i].iov_base, n);
+		p += n;
+		nleft -= n;
+		if (nleft == 0)
+			break;
+	}
+
+	return SSL_write(ssl, buf, p - buf);
+}
+
 int CommTarget::init(const struct sockaddr *addr, socklen_t addrlen,
 					 int connect_timeout, int response_timeout)
 {
@@ -178,6 +208,13 @@ int CommMessageIn::feedback(const void *buf, size_t size)
 	return ret;
 }
 
+void CommMessageIn::renew()
+{
+	CommSession *session = this->entry->session;
+	session->timeout = -1;
+	session->begin_time.tv_nsec = -1;
+}
+
 int CommService::init(const struct sockaddr *bind_addr, socklen_t addrlen,
 					  int listen_timeout, int response_timeout)
 {
@@ -412,7 +449,7 @@ int Communicator::send_message_sync(struct iovec vectors[], int cnt,
 		}
 		else if (vectors->iov_len > 0)
 		{
-			n = SSL_write(entry->ssl, vectors->iov_base, vectors->iov_len);
+			n = __ssl_writev(entry->ssl, vectors, cnt);
 			if (n <= 0)
 				return cnt;
 		}
